/*
 * Copyright (c) 2013 - 2014, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 ///////////////////////////////////////////////////////////////////////////////
//  Includes
///////////////////////////////////////////////////////////////////////////////

// Standard C Included Files
#include <stdio.h>
// SDK Included Files
#include "board.h"
#include "gpio_pins.h"
#include "fsl_debug_console.h"

///////////////////////////////////////////////////////////////////////////////
// Variables
///////////////////////////////////////////////////////////////////////////////

volatile bool isButtonPress = false;

//uint32_t output_pinname = kGpioLED1;                                                                 
uint32_t output_pinname = GPIO_MAKE_PIN(GPIOE_IDX, 3);			// either 3 or 4
uint32_t input_pinname = GPIO_MAKE_PIN(GPIOA_IDX, 1);			// Switch 12 -- the lower-right hand side one

enum lcd_pins {
	LCD_RS  = GPIO_MAKE_PIN(GPIOC_IDX, 13),
	LCD_RW  = GPIO_MAKE_PIN(GPIOC_IDX, 16),
	LCD_E   = GPIO_MAKE_PIN(GPIOC_IDX, 17),
	LCD_DB0 = GPIO_MAKE_PIN(GPIOD_IDX, 0),
	LCD_DB1 = GPIO_MAKE_PIN(GPIOD_IDX, 1),
	LCD_DB2 = GPIO_MAKE_PIN(GPIOD_IDX, 2),
	LCD_DB3 = GPIO_MAKE_PIN(GPIOD_IDX, 3),
	LCD_DB4 = GPIO_MAKE_PIN(GPIOD_IDX, 4),
	LCD_DB5 = GPIO_MAKE_PIN(GPIOD_IDX, 5),
	LCD_DB6 = GPIO_MAKE_PIN(GPIOD_IDX, 6),
	LCD_DB7 = GPIO_MAKE_PIN(GPIOD_IDX, 7)
};


////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////

void my_hardware_init(void) {

  /* enable clock for PORTs */
  CLOCK_SYS_EnablePortClock(PORTA_IDX);
  CLOCK_SYS_EnablePortClock(PORTB_IDX);
  CLOCK_SYS_EnablePortClock(PORTC_IDX);
  CLOCK_SYS_EnablePortClock(PORTD_IDX);
  CLOCK_SYS_EnablePortClock(PORTE_IDX);

  /* Init board clock */
  //BOARD_ClockInit();			// IMPORTANT -- THIS HAS BEEN COMMENTED OUT
  //dbg_uart_init();
}


// since OSA_TimeDelay() isn't working, we'll burn CPU instead
long delay(int repetitions) {
	long f = 1000000000;
	for (int i=0; i<repetitions; i++) {
		f = f / 46;
	}
	return f;
}


// since OSA_TimeDelay() isn't working, we'll burn CPU instead
long delay40uS() {
	delay(20);
}


// this is required before any other communication
int LCD_set_4bit_mode() {
	GPIO_DRV_ClearPinOutput(LCD_RW);
	GPIO_DRV_ClearPinOutput(LCD_RS);

	GPIO_DRV_SetPinOutput(  LCD_E);

	GPIO_DRV_SetPinOutput(  LCD_DB4);
	GPIO_DRV_SetPinOutput(  LCD_DB5);
	GPIO_DRV_ClearPinOutput(LCD_DB6);
	GPIO_DRV_ClearPinOutput(LCD_DB7);

	GPIO_DRV_ClearPinOutput(LCD_E);
}


// the 4bit-mode version of LCD_write_10bits()
int LCD_4B_write_10bits(int val) {
	int retval = 0;

	GPIO_DRV_WritePinOutput(LCD_RW,  ((val >> 8) & 0x1));
	GPIO_DRV_WritePinOutput(LCD_RS,  ((val >> 9) & 0x1));

	if (!((val >> 9) & 0x1)) {			// LCD_RW -- high means "read", low means "write"
		// ============ write ==============
		GPIO_DRV_SetPinOutput(LCD_E);

		GPIO_DRV_WritePinOutput(LCD_DB4, ((val >> 4) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB5, ((val >> 5) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB6, ((val >> 6) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB7, ((val >> 7) & 0x1));

		GPIO_DRV_ClearPinOutput(LCD_E);
		delay(2);
		GPIO_DRV_SetPinOutput(LCD_E);

		GPIO_DRV_WritePinOutput(LCD_DB4, ((val >> 0) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB5, ((val >> 1) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB6, ((val >> 2) & 0x1));
		GPIO_DRV_WritePinOutput(LCD_DB7, ((val >> 3) & 0x1));

		GPIO_DRV_ClearPinOutput(LCD_E);
	} else {
		// ============ read ==============
		GPIO_DRV_SetPinDir(LCD_DB4, kGpioDigitalInput);
		GPIO_DRV_SetPinDir(LCD_DB5, kGpioDigitalInput);
		GPIO_DRV_SetPinDir(LCD_DB6, kGpioDigitalInput);
		GPIO_DRV_SetPinDir(LCD_DB7, kGpioDigitalInput);

		GPIO_DRV_SetPinOutput(LCD_E);

		retval =
			(GPIO_DRV_ReadPinInput(LCD_DB7) << 7) +
			(GPIO_DRV_ReadPinInput(LCD_DB6) << 6) +
			(GPIO_DRV_ReadPinInput(LCD_DB5) << 5) +
			(GPIO_DRV_ReadPinInput(LCD_DB4) << 4);

		GPIO_DRV_ClearPinOutput(LCD_E);
		delay(2);
		GPIO_DRV_SetPinOutput(LCD_E);

		retval +=
			(GPIO_DRV_ReadPinInput(LCD_DB7) << 3) +
			(GPIO_DRV_ReadPinInput(LCD_DB6) << 2) +
			(GPIO_DRV_ReadPinInput(LCD_DB5) << 1) +
			(GPIO_DRV_ReadPinInput(LCD_DB4) << 0);

		GPIO_DRV_SetPinDir(LCD_DB4, kGpioDigitalOutput);
		GPIO_DRV_SetPinDir(LCD_DB5, kGpioDigitalOutput);
		GPIO_DRV_SetPinDir(LCD_DB6, kGpioDigitalOutput);
		GPIO_DRV_SetPinDir(LCD_DB7, kGpioDigitalOutput);

		GPIO_DRV_ClearPinOutput(LCD_E);
	}

	return retval;
}


// the 4bit-mode version of LCD_display_on()
void LCD_4B_display_on() {
	int i;
	for (int i=0; i<100; i++) {
		delay40uS();
	}
	//OSA_TimeDelay(40);
	LCD_set_4bit_mode();
	//OSA_TimeDelay(2);
	delay40uS();
	LCD_4B_write_10bits(0x028);		// function set -- num lines = 2 -- font size = 5x8 -- interface =  8bit
	//OSA_TimeDelay(2);
	delay40uS();
	LCD_4B_write_10bits(0x028);		// function set
	//OSA_TimeDelay(2);
	delay40uS();
	LCD_4B_write_10bits(0x00F);		// display on/off -- entire display on -- cursor on -- cursor position on
	//OSA_TimeDelay(2);
	delay40uS();
	LCD_4B_write_10bits(0x00F);		// repeat
	//OSA_TimeDelay(2);
	delay40uS();
	LCD_4B_clear_screen();
}


void LCD_4B_clear_screen() {
	LCD_4B_write_10bits(0x001);		// display clear
	delay(800);						// sleep 1.5 ms
}


// write a character to the screen
void LCD_4B_write_character(uint32_t ddram_addr, uint32_t value) {
	LCD_4B_write_10bits(0x080 | (ddram_addr & 0x7F));		// set DDRAM address
	delay40uS();

	LCD_4B_write_10bits(0x200 | (value & 0xFF));			// write data to RAM
	delay40uS();
}


// read what character is at a particular location on the screen
uint32_t LCD_4B_read_character(uint32_t ddram_addr) {
	LCD_4B_write_10bits(0x080 | (ddram_addr & 0x7F));		// set DDRAM address
	delay40uS();

	uint32_t retval = LCD_4B_write_10bits(0x300);			// write data to RAM}
	delay40uS();

	return retval;
}


/*!
 * @brief Uses a switch to controll a LED.
 *
 * This function toogles LED1 when press the switch.
 */
int main(void)
{
    // Define gpio input pin config structure SW.
    gpio_input_pin_user_config_t inputPin[] = {
		{
			.pinName                       = LCD_DB0,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB1,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB2,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB3,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB4,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB5,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB6,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
		{
			.pinName                       = LCD_DB7,
			.config.isPullEnable           = true,
			.config.pullSelect             = kPortPullUp,
			.config.isPassiveFilterEnabled = false,
			.config.interrupt              = kPortIntFallingEdge,
		},
        {
            .pinName = GPIO_PINS_OUT_OF_RANGE,
        }
    };
    
    gpio_input_pin_user_config_t inputPin_empty[] = {
		{
			.pinName = GPIO_PINS_OUT_OF_RANGE,
		}
    };


    // Define gpio output pin config structure LED1.
    gpio_output_pin_user_config_t outputPin[] = {
		{
			.pinName              = LCD_RS,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_RW,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_E,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB0,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB1,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB2,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB3,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB4,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB5,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB6,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
		{
			.pinName              = LCD_DB7,
			.config.outputLogic   = 0,
			.config.slewRate      = kPortFastSlewRate,
		},
        {
            .pinName = GPIO_PINS_OUT_OF_RANGE,
        }
    };

    // Init hardware
    //hardware_init();
    my_hardware_init();

    //GPIO_DRV_Init(inputPin, outputPin);
    GPIO_DRV_Init(inputPin_empty, outputPin);

	GPIO_DRV_SetPinDir(LCD_DB0, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB1, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB2, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB3, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB4, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB5, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB6, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_DB7, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_RS, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_RW, kGpioDigitalOutput);
	GPIO_DRV_SetPinDir(LCD_E,  kGpioDigitalOutput);

	LCD_4B_display_on();

	/* LCD_4B_write_character(1, 0x15);
	LCD_4B_clear_screen();
	LCD_4B_write_character(1, 0x14);
	LCD_4B_clear_screen();
	LCD_4B_write_character(1, 0x13);
	LCD_4B_clear_screen();
	LCD_4B_write_character(1, 0x12);
	LCD_4B_clear_screen();
	LCD_4B_write_character(1, 0x11);
	LCD_4B_clear_screen();
	LCD_4B_write_character(1, 0x10);*/

	LCD_4B_write_character(0x40, 0x14);
	LCD_4B_write_character(0x40, 0x15);
	LCD_4B_write_character(0x40, 0x16);
	LCD_4B_write_character(0x40, 0x17);
	LCD_4B_write_character(0x41, 0x17);
	LCD_4B_write_character(0x42, 0x17);
	LCD_4B_write_character(0x43, 0x17);
	LCD_4B_write_character(0x44, 0x17);



	int t = LCD_4B_read_character(0);
	t = LCD_4B_read_character(1);
	t = LCD_4B_read_character(2);
	t = LCD_4B_read_character(0x40);

	//LCD_4B_write_character(0, 'a');
	//LCD_4B_write_character(1, 'b');
	//LCD_4B_write_character(2, 'c');


	delay40uS();
}



/*******************************************************************************
 * EOF
 ******************************************************************************/
